@@ -73,12 +73,12 @@ Board2chCompati::~Board2chCompati()
 //
 // キャッシュのファイル名が正しいか
 //
-bool Board2chCompati::is_valid( const std::string& filename )
+const bool Board2chCompati::is_valid( const std::string& filename )
 {
     if( filename.find( get_ext() ) == std::string::npos ) return false;
     if( filename.length() - filename.rfind( get_ext() ) != get_ext().length() ) return false;
 
-    unsigned int dig, n;
+    size_t dig, n;
     MISC::str_to_uint( filename.c_str(), dig, n );
     if( dig != n ) return false;
     if( dig == 0 ) return false;
@@ -90,7 +90,7 @@ bool Board2chCompati::is_valid( const std::string& filename )
 //書き込み用クッキー作成
 const std::string Board2chCompati::cookie_for_write()
 {
-    std::list< std::string > list_cookies = BoardBase::list_cookies_for_write();
+    const std::list< std::string > list_cookies = BoardBase::list_cookies_for_write();
     if( list_cookies.empty() ) return std::string();
 
 #ifdef _DEBUG
@@ -98,71 +98,88 @@ const std::string Board2chCompati::cookie_for_write()
 #endif
 
     JDLIB::Regex regex;
+    const size_t offset = 0;
+    const bool icase = false;
+    const bool newline = true;
+    const bool usemigemo = false;
+    const bool wchar = false;
 
-    std::string str_expire;
-    std::string query_expire = "expires=([^;]*)";
+    std::string cookie_expire;
+    std::string cookie_path;
+    std::string cookie_pon;
+    std::string cookie_hap = get_hap();
+    std::string cookie_name;
+    std::string cookie_mail;
 
-    std::string str_path;
-    std::string query_path = "path=([^;]*)";
+    const std::string query_expire = "expires=([^;]*)";
+    const std::string query_path = "path=([^;]*)";
+    const std::string query_pon = "PON=([^;]*)?";
+    const std::string query_hap = "HAP=([^;]*)?";
+    const std::string query_name = "NAME=([^;]*)?";
+    const std::string query_mail = "MAIL=([^;]*)?";
 
     bool use_pon = false;
-    std::string str_pon;
-    std::string query_pon = "PON=([^;]*)?";
-
+    bool use_hap = ! cookie_hap.empty();
     bool use_name = false;
-    std::string str_name;
-    std::string query_name = "NAME=([^;]*)?";
-
     bool use_mail = false;
-    std::string str_mail;
-    std::string query_mail = "MAIL=([^;]*)?";
 
-    std::list< std::string >::iterator it = list_cookies.begin();
+    std::list< std::string >::const_iterator it = list_cookies.begin();
 
     // expire と path は一つ目のcookieから取得
-    if( regex.exec( query_expire, (*it) ) ) str_expire = regex.str( 1 );
-    if( regex.exec( query_path, (*it) ) ) str_path = regex.str( 1 );
+    if( regex.exec( query_expire, (*it), offset, icase, newline, usemigemo, wchar ) ) cookie_expire = regex.str( 1 );
+    if( regex.exec( query_path, (*it), offset, icase, newline, usemigemo, wchar ) ) cookie_path = regex.str( 1 );
 
-    // その他は iterateして取得
     for( ; it != list_cookies.end(); ++it ){
 
-        std::string tmp_cookie = MISC::Iconv( (*it), get_charset(), "UTF-8" );
+        const std::string tmp_cookie = MISC::Iconv( (*it), get_charset(), "UTF-8" );
 
 #ifdef _DEBUG
         std::cout << tmp_cookie << std::endl;
 #endif
 
-        if( regex.exec( query_pon, tmp_cookie ) ){
+        if( regex.exec( query_pon, tmp_cookie, offset, icase, newline, usemigemo, wchar ) ){
             use_pon = true;
-            str_pon = regex.str( 1 );
+            cookie_pon = regex.str( 1 );
         }
-        if( regex.exec( query_name, tmp_cookie ) ){
+        if( ! use_hap && regex.exec( query_hap, tmp_cookie, offset, icase, newline, usemigemo, wchar ) ){
+            use_hap = true;
+            cookie_hap = regex.str( 1 );
+        }
+        if( regex.exec( query_name, tmp_cookie, offset, icase, newline, usemigemo, wchar ) ){
             use_name = true;
-            str_name = MISC::charset_url_encode( regex.str( 1 ), get_charset() );
+            cookie_name = MISC::charset_url_encode( regex.str( 1 ), get_charset() );
         }
-        if( regex.exec( query_mail, tmp_cookie ) ){
+        if( regex.exec( query_mail, tmp_cookie, offset, icase, newline, usemigemo, wchar ) ){
             use_mail = true;
-            str_mail = MISC::charset_url_encode( regex.str( 1 ), get_charset() );
+            cookie_mail = MISC::charset_url_encode( regex.str( 1 ), get_charset() );
         }
     }
 
+    // PONを取得していないときはHAPを送らない
+    if( ! use_pon || cookie_pon.empty() ){
+        use_hap = false;
+        cookie_hap = std::string();
+    }
+
 #ifdef _DEBUG
-    std::cout << "expire = " << str_expire << std::endl
-              << "path = " << str_path << std::endl    
-              << "pon = " << str_pon << std::endl
-              << "name = " << str_name << std::endl
-              << "mail = " << str_mail << std::endl;
+    std::cout << "expire = " << cookie_expire << std::endl
+              << "path = " << cookie_path << std::endl    
+              << "pon = " << cookie_pon << " " << use_pon << std::endl
+              << "hap = " << cookie_hap << " " << use_hap << std::endl
+              << "name = " << cookie_name << " " << use_name << std::endl
+              << "mail = " << cookie_mail << " " << use_mail << std::endl;
 #endif    
 
     std::string cookie;
 
-    if( use_name ) cookie += "NAME=" + str_name + "; ";
-    if( use_mail ) cookie += "MAIL=" + str_mail + "; ";
-    if( use_pon ) cookie += "PON=" + str_pon + "; ";
+    if( use_pon ) cookie += "PON=" + cookie_pon + "; ";
+    if( use_hap ) cookie += "HAP=" + cookie_hap + "; ";
+    if( use_name ) cookie += "NAME=" + cookie_name + "; ";
+    if( use_mail ) cookie += "MAIL=" + cookie_mail + "; ";
 
     if( cookie.empty() ) return std::string();
 
-    cookie += "expires=" + str_expire + "; path=" + str_path;
+    cookie += "expires=" + cookie_expire + "; path=" + cookie_path;
 
 #ifdef _DEBUG
     std::cout << "cookie = " << cookie << std::endl;
@@ -185,11 +202,15 @@ void Board2chCompati::analyze_keyword_for_write( const std::string& html )
 
     JDLIB::Regex regex;
     size_t offset = 0;
+    const bool icase = true; // 大文字小文字区別しない
+    const bool newline = false;  // . に改行をマッチさせる
+    const bool usemigemo = false;
+    const bool wchar = false;
 
     for(;;){
 
         // <input type=hidden> のタグを解析して name と value を取得
-        if( ! regex.exec( "<input +type=hidden +name=([^ ]*) +value=([^>]*)>", html, offset, true, false ) ) break;
+        if( ! regex.exec( "<input +type=hidden +name=([^ ]*) +value=([^>]*)>", html, offset, icase, newline, usemigemo, wchar ) ) break;
 
         offset = html.find( regex.str( 0 ) );
 
@@ -308,16 +329,16 @@ ArticleBase* Board2chCompati::append_article( const std::string& datbase, const
 //
 // subject.txt から Aarticle のリストにアイテムを追加・更新
 //
-void Board2chCompati::parse_subject( const char* str_subject_txt, const bool is_online )
+void Board2chCompati::parse_subject( const char* str_subject_txt )
 {
 #ifdef _DEBUG
     std::cout << "Board2chCompati::parse_subject\n";
 #endif 
 
-    const int max_subject = 512;
+    // 半角で 1024 文字を越えると Pango-WARNING **: Invalid UTF-8 string passed to pango_layout_set_text() が表示されるので注意
+    const int max_subject = 1024;  
+
     const char* pos = str_subject_txt;
-    char str_tmp[ max_subject ];
-    const std::string datbase = url_datbase();
 
     while( *pos != '\0' ){
         
@@ -327,23 +348,39 @@ void Board2chCompati::parse_subject( const char* str_subject_txt, const bool is_
         int lng_subject = 0;
         char str_num[ 16 ];
 
+        while( *pos == ' ' ) ++pos;
+
         // datのID取得
         str_id_dat = pos;
-        while( *pos != '<' && *pos != '\0' && *pos != '\n' ) { ++pos; ++lng_id_dat; }
+        while( *pos != ' ' && *pos != '<' && *pos != '\0' && *pos != '\n' ) { ++pos; ++lng_id_dat; }
         
         // 壊れてる
-        if( *pos == '\0' ) break;
+        if( *pos == '\0' ){
+            MISC::ERRMSG( "subject.txt is broken" );
+            break;
+        }
         if( *pos == '\n' ) { ++pos; continue; }
 
+        while( *pos != '\0' && *pos != '<' ) ++pos;
+        if( *pos != '\0' ) ++pos;
+        if( *pos != '>' ){
+            MISC::ERRMSG( "subject.txt is broken" );
+            break;
+        }
+
         // subject取得
-        pos += 2;
+        bool exist_amp = false;
+        ++pos;
         str_subject = pos;
-        while( *pos != '\0' && *pos != '\n' ) ++pos;
+        while( *pos != '\0' && *pos != '\n' ){
+            if( *pos == '&' ) exist_amp = true;
+            ++pos;
+        }
         --pos;
-        while( *pos != '(' && *pos != '\n' && pos != str_subject_txt ) --pos;
+        while( *pos != '(' && *pos != '\n' && pos != str_subject ) --pos;
         
         // 壊れてる
-        if( *pos == '\n' || pos == str_subject_txt ){
+        if( *pos == '\n' || pos == str_subject ){
             MISC::ERRMSG( "subject.txt is broken" );
             break;
         }
@@ -352,9 +389,13 @@ void Board2chCompati::parse_subject( const char* str_subject_txt, const bool is_
         // レス数取得
         ++pos;
         int i = 0;
-        while( *pos != ')' && *pos != '\0' && *pos != '\n' && i < 16 ) str_num[ i++ ] = *( pos++ );
+        while( '0' <= *pos && *pos <= '9' && i < 16 ) str_num[ i++ ] = *( pos++ );
 
         // 壊れてる
+        if( i == 0 ){
+            MISC::ERRMSG( "subject.txt is broken (res)" );
+            break;
+        }
         if( *pos == '\0' ) break;
         if( *pos == '\n' ) { ++pos; continue; }
 
@@ -362,32 +403,54 @@ void Board2chCompati::parse_subject( const char* str_subject_txt, const bool is_
         ++pos;
 
         // id, subject, number 取得
-        memcpy( str_tmp, str_id_dat, lng_id_dat );
-        str_tmp[ lng_id_dat ] = '\0';
-        std::string id = MISC::remove_space( str_tmp );
+        ARTICLE_INFO artinfo;
 
-        memcpy( str_tmp, str_subject, lng_subject );
-        if( str_tmp[ lng_subject-1 ] == ' ' ) lng_subject--;  // 2chのsubject.txtは()の前に空白が一つ入る
-        str_tmp[ lng_subject ] = '\0';
-        std::string subject = str_tmp;
-        subject = MISC::replace_str( subject, "&lt;", "<" );
-        subject = MISC::replace_str( subject, "&gt;", ">" );
+        artinfo.id.assign( str_id_dat, lng_id_dat );
+
+        if( str_subject[ lng_subject-1 ] == ' ' ){
+            lng_subject--;  // 2chのsubject.txtは()の前に空白が一つ入る
+        }
+        artinfo.subject.assign( str_subject, lng_subject );
+
+        if( exist_amp ){
+            artinfo.subject = MISC::replace_str( artinfo.subject, "&lt;", "<" );
+            artinfo.subject = MISC::replace_str( artinfo.subject, "&gt;", ">" );
+        }
         
-        int number = atol( str_num );
+        artinfo.number = atol( str_num );
+
+        get_list_artinfo().push_back( artinfo );
 
 #ifdef _DEBUG
-        std::cout << pos - str_subject_txt << " " << lng_subject << " id = " << id << " num = " << number;
-        std::cout << " : " << subject << std::endl;
+        std::cout << "pos = " << ( pos - str_subject_txt ) << " lng = " << lng_subject << " id = " << artinfo.id << " num = " << artinfo.number;
+        std::cout << " : " << artinfo.subject << std::endl;
 #endif
+    }
+}
 
+
+void Board2chCompati::regist_article( const bool is_online )
+{
+    if( ! get_list_artinfo().size() ) return;
+
+#ifdef _DEBUG
+    std::cout << "Board2chCompati::regist_article size = " << get_list_artinfo().size() << std::endl;
+#endif 
+
+    const std::string datbase = url_datbase();
+
+    for( unsigned int i = 0; i < get_list_artinfo().size(); ++i ){
+
+        const ARTICLE_INFO& artinfo = get_list_artinfo()[ i ];
+        
         // DBに登録されてるならarticle クラスの情報更新
-        ArticleBase* article = get_article( datbase, id );
+        ArticleBase* article = get_article( datbase, artinfo.id );
 
         // DBにないなら新規に article クラスをDBに登録
         //
         // なお BoardBase::receive_finish() のなかで append_all_article_in_cache() が既に呼び出されているため
         // DBに無いということはキャッシュに無いということ。よって append_article()の呼出に cached = false　を指定する
-        if( article->empty() ) article = append_article( datbase, id,
+        if( article->empty() ) article = append_article( datbase, artinfo.id,
                                                          false // キャッシュ無し
             );
 
@@ -404,8 +467,8 @@ void Board2chCompati::parse_subject( const char* str_subject_txt, const bool is_
             article->read_info();
 
             // 情報ファイルが無い場合もあるのでsubject.txtから取得したサブジェクト、レス数を指定しておく
-            article->set_subject( subject );
-            article->set_number( number, is_online );
+            article->set_subject( artinfo.subject );
+            article->set_number( artinfo.number, is_online );
 
             // 情報ファイル読み込み後にステータスが変わることがあるので、もう一度
             // ステータスをDAT落ち状態から通常状態に変更
@@ -425,7 +488,7 @@ const std::string Board2chCompati::localrule()
 {
     if( m_ruleloader ){
         if( m_ruleloader->is_loading() ) return "ロード中です";
-        else if( m_ruleloader->get_code() == HTTP_OK ){
+        else if( m_ruleloader->get_code() == HTTP_OK || m_ruleloader->get_code() == HTTP_REDIRECT || m_ruleloader->get_code() == HTTP_MOVED_PERM ){
             if( m_ruleloader->get_data().empty() ) return "ローカルルールはありません";
             else return m_ruleloader->get_data();
         }
@@ -440,7 +503,7 @@ const std::string Board2chCompati::settingtxt()
 {
     if( m_settingloader ){
         if( m_settingloader->is_loading() ) return "ロード中です";
-        else if( m_settingloader->get_code() == HTTP_OK ){
+        else if( m_settingloader->get_code() == HTTP_OK || m_settingloader->get_code() == HTTP_REDIRECT || m_settingloader->get_code() == HTTP_MOVED_PERM ){
             if( m_settingloader->get_data().empty() ) return "SETTING.TXTはありません";
             else return m_settingloader->get_data();
         }
@@ -478,6 +541,15 @@ const int Board2chCompati::message_count()
 }    
 
 
+const std::string Board2chCompati::get_unicode()
+{
+    if( m_settingloader
+        && m_settingloader->get_code() == HTTP_OK ) return m_settingloader->get_unicode();
+
+    return BoardBase::get_unicode();
+}
+
+
 //
 // ローカルルールとSETTING.TXTをキャッシュから読み込む
 //
